module  bus_sync #(
  parameter DW = 8
)
(
input                src_clk     ,
input                src_rst_n   ,
input  [DW-1:0]      src_bus     ,
input                dst_clk     ,
input                dst_rst_n   ,
output  reg [DW-1:0] dst_bus     

);

reg           src_req      ;
wire          src_req_sync ;
reg           dst_ack      ;
wire          dst_ack_sync ;
wire          sync_ok      ; 
reg  [DW-1:0] src_bus_lock ;

//===================>>>>===========================
always @(posedge src_clk or negedge src_rst_n)
begin
  if(!src_rst_n)
     src_req <=  1'b0;
  else if(src_req)
     src_req <=  ~dst_ack_sync ;  
  else if(sync_ok)
     src_req <=  1'b1;
end

//2 stage
bit_sync #(
  .DW   ( 1 ),
  .STAGE( 3 )
) bit_sync_u0(
     .clk   (dst_clk        ) ,
     .rst_n (dst_rst_n      ) ,
     .din   (src_req        ) ,
     .dout  (src_req_sync   )

);

always @(posedge src_clk or negedge src_rst_n)
begin
  if(!src_rst_n)
     src_bus_lock <= {DW{1'b0}};
  else if(sync_ok)
     src_bus_lock <=  src_bus ;   
end
assign sync_ok = ((!dst_ack_sync) && (!src_req)) ? 1'b1 : 1'b0 ;

//===================<<<<===========================

always @(posedge dst_clk or negedge dst_rst_n)
begin
  if(!dst_rst_n)
     dst_ack <=  1'b0;
  else 
     dst_ack <=  src_req_sync ;   
end
//2 stage
bit_sync #(
  .DW   ( 1 ),
  .STAGE( 3 )
)  bit_sync_u1(
     .clk   (src_clk        ) ,
     .rst_n (src_rst_n      ) ,
     .din   (dst_ack        ) ,
     .dout  (dst_ack_sync   )

);

always @(posedge dst_clk or negedge dst_rst_n)
begin
  if(!dst_rst_n)
     dst_bus <= {DW{1'b0}};
  else if(src_req_sync)
     dst_bus <=  src_bus_lock ;   
end


endmodule

